@@ -909,6 +909,8 @@ class ClientConstructor:
         error_sampler=None,  # type: Optional[Callable[[Event, Hint], Union[float, bool]]]
         enable_db_query_source=True,  # type: bool
         db_query_source_threshold_ms=100,  # type: int
+        enable_http_request_source=False,  # type: bool
+        http_request_source_threshold_ms=100,  # type: int
         spotlight=None,  # type: Optional[Union[bool, str]]
         cert_file=None,  # type: Optional[str]
         key_file=None,  # type: Optional[str]
@@ -1264,6 +1266,13 @@ class ClientConstructor:
 
             The query location will be added to the query for queries slower than the specified threshold.
 
+        :param enable_http_request_source: When enabled, the source location will be added to outgoing HTTP requests.
+
+        :param http_request_source_threshold_ms: The threshold in milliseconds for adding the source location to an
+            outgoing HTTP request.
+
+            The request location will be added to the request for requests slower than the specified threshold.
+
         :param custom_repr: A custom `repr <https://docs.python.org/3/library/functions.html#repr>`_ function to run
             while serializing an object.
 
@@ -1339,4 +1348,4 @@ DEFAULT_OPTIONS = _get_default_options()
 del _get_default_options
 
 
-VERSION = "2.41.0"
+VERSION = "2.42.0"
