@@ -409,7 +409,7 @@ class SentryLogsHandler(_BaseHandler):
             attrs["logger.name"] = record.name
 
         # noinspection PyProtectedMember
-        client._capture_experimental_log(
+        client._capture_log(
             {
                 "severity_text": otel_severity_text,
                 "severity_number": otel_severity_number,
